@@ -238,7 +238,9 @@ void x265_setup_primitives(x265_param *param)
             primitives.cu[i].intra_pred_allangs = NULL;
 
 #if ENABLE_ASSEMBLY
+#if X265_ARCH_X86
         setupInstrinsicPrimitives(primitives, param->cpuid);
+#endif
         setupAssemblyPrimitives(primitives, param->cpuid);
 #endif
 
@@ -249,7 +251,7 @@ void x265_setup_primitives(x265_param *param)
 }
 }
 
-#if ENABLE_ASSEMBLY
+#if ENABLE_ASSEMBLY && X265_ARCH_X86
 /* these functions are implemented in assembly. When assembly is not being
  * compiled, they are unnecessary and can be NOPs */
 #else
@@ -258,7 +260,10 @@ int PFX(cpu_cpuid_test)(void) { return 0; }
 void PFX(cpu_emms)(void) {}
 void PFX(cpu_cpuid)(uint32_t, uint32_t *eax, uint32_t *, uint32_t *, uint32_t *) { *eax = 0; }
 void PFX(cpu_xgetbv)(uint32_t, uint32_t *, uint32_t *) {}
+
+#if X265_ARCH_ARM == 0
 void PFX(cpu_neon_test)(void) {}
 int PFX(cpu_fast_neon_mrc_test)(void) { return 0; }
+#endif // X265_ARCH_ARM
 }
 #endif
