@@ -282,6 +282,9 @@ void x265_picture_init(x265_param *param, x265_picture *pic)
     pic->colorSpace = param->internalCsp;
     pic->forceqp = X265_QP_AUTO;
     pic->quantOffsets = NULL;
+    pic->userSEI.payloads = NULL;
+    pic->userSEI.numPayloads = 0;
+
     if (param->analysisMode)
     {
         uint32_t widthInCU       = (param->sourceWidth  + g_maxCUSize - 1) >> g_maxLog2CUSize;
