@@ -27,7 +27,7 @@
 #include "output.h"
 #include <fstream>
 
-namespace x265 {
+namespace X265_NS {
 // private x265 namespace
 
 class Y4MOutput : public ReconFile
