@@ -29,7 +29,7 @@
 
 #include <fstream>
 
-namespace x265 {
+namespace X265_NS {
 // private x265 namespace
 
 class YUVOutput : public ReconFile
